"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfigurableDateString = exports.getConfigureDateISOString = exports.relativeDateToStatic = exports.directionOrDefault = exports.pluralizeDuration = exports.depluralizeDuration = exports.isPluralDuration = exports.durationOrDefault = exports.parseRelativeDate = exports.isStaticDate = exports.isRelativeDate = exports.relativeConfigurableDate = exports.staticConfigurableDate = void 0;
const bf_types_1 = require("bf-types");
/* ~~~ Data Generators ~~~ */
function staticConfigurableDate(date, time) {
    return {
        type: bf_types_1.ConfigurableDateType.STATIC,
        value: new Date(`${date} ${time}`).toISOString(),
    };
}
exports.staticConfigurableDate = staticConfigurableDate;
function relativeConfigurableDate(quantifier, duration, direction) {
    return {
        type: bf_types_1.ConfigurableDateType.RELATIVE,
        value: `${quantifier} ${duration} ${direction}`,
    };
}
exports.relativeConfigurableDate = relativeConfigurableDate;
/* ~~~ Data Testers ~~~ */
function isRelativeDate(date) {
    return date.type === bf_types_1.ConfigurableDateType.RELATIVE;
}
exports.isRelativeDate = isRelativeDate;
function isStaticDate(date) {
    return date.type === bf_types_1.ConfigurableDateType.STATIC;
}
exports.isStaticDate = isStaticDate;
/* ~~~ Helper Functions ~~~ */
function parseRelativeDate(date) {
    const chunks = date.value.split(/\s+/).filter((chunk) => Boolean(chunk));
    const quantifier = Number(chunks[0]);
    const duration = durationOrDefault(chunks[1]);
    const direction = directionOrDefault(chunks.slice(2).join(' '));
    return {
        quantifier,
        duration,
        direction,
    };
}
exports.parseRelativeDate = parseRelativeDate;
function durationOrDefault(duration) {
    switch (duration) {
        case bf_types_1.RelativeDateDuration.DISABLED:
        case bf_types_1.RelativeDateDuration.DAY:
        case bf_types_1.RelativeDateDuration.DAYS:
        case bf_types_1.RelativeDateDuration.WEEK:
        case bf_types_1.RelativeDateDuration.WEEKS:
        case bf_types_1.RelativeDateDuration.MONTH:
        case bf_types_1.RelativeDateDuration.MONTHS:
        case bf_types_1.RelativeDateDuration.YEAR:
        case bf_types_1.RelativeDateDuration.YEARS: {
            return duration;
        }
    }
    return bf_types_1.RelativeDateDuration.WEEK;
}
exports.durationOrDefault = durationOrDefault;
function isPluralDuration(duration) {
    switch (duration) {
        case bf_types_1.RelativeDateDuration.DAYS:
        case bf_types_1.RelativeDateDuration.WEEKS:
        case bf_types_1.RelativeDateDuration.MONTHS:
        case bf_types_1.RelativeDateDuration.YEARS: {
            return true;
        }
    }
    return false;
}
exports.isPluralDuration = isPluralDuration;
function depluralizeDuration(duration) {
    switch (duration) {
        case bf_types_1.RelativeDateDuration.DISABLED: {
            return bf_types_1.RelativeDateDuration.DISABLED;
        }
        case bf_types_1.RelativeDateDuration.DAYS: {
            return bf_types_1.RelativeDateDuration.DAY;
        }
        case bf_types_1.RelativeDateDuration.WEEKS: {
            return bf_types_1.RelativeDateDuration.WEEK;
        }
        case bf_types_1.RelativeDateDuration.MONTHS: {
            return bf_types_1.RelativeDateDuration.MONTH;
        }
        case bf_types_1.RelativeDateDuration.YEARS: {
            return bf_types_1.RelativeDateDuration.YEAR;
        }
    }
    return duration;
}
exports.depluralizeDuration = depluralizeDuration;
function pluralizeDuration(duration) {
    switch (duration) {
        case bf_types_1.RelativeDateDuration.DISABLED: {
            return bf_types_1.RelativeDateDuration.DISABLED;
        }
        case bf_types_1.RelativeDateDuration.DAY: {
            return bf_types_1.RelativeDateDuration.DAYS;
        }
        case bf_types_1.RelativeDateDuration.WEEK: {
            return bf_types_1.RelativeDateDuration.WEEKS;
        }
        case bf_types_1.RelativeDateDuration.MONTH: {
            return bf_types_1.RelativeDateDuration.MONTHS;
        }
        case bf_types_1.RelativeDateDuration.YEAR: {
            return bf_types_1.RelativeDateDuration.YEARS;
        }
    }
    return duration;
}
exports.pluralizeDuration = pluralizeDuration;
function directionOrDefault(direction) {
    switch (direction) {
        case bf_types_1.RelativeDateDirection.AFTER_TODAY:
        case bf_types_1.RelativeDateDirection.BEFORE_TODAY:
        case bf_types_1.RelativeDateDirection.TODAY: {
            return direction;
        }
    }
    return bf_types_1.RelativeDateDirection.AFTER_TODAY;
}
exports.directionOrDefault = directionOrDefault;
function relativeDateToStatic(relativeDate) {
    const date = new Date();
    date.setHours(0);
    date.setMinutes(0);
    date.setSeconds(0);
    date.setMilliseconds(0);
    const { quantifier, duration, direction } = parseRelativeDate(relativeDate);
    switch (duration) {
        case bf_types_1.RelativeDateDuration.DAY:
        case bf_types_1.RelativeDateDuration.DAYS: {
            switch (direction) {
                case bf_types_1.RelativeDateDirection.BEFORE_TODAY: {
                    const currentDay = date.getDate();
                    date.setDate(currentDay - quantifier);
                    break;
                }
                case bf_types_1.RelativeDateDirection.AFTER_TODAY: {
                    const currentDay = date.getDate();
                    date.setDate(currentDay + quantifier);
                    break;
                }
            }
            break;
        }
        case bf_types_1.RelativeDateDuration.WEEK:
        case bf_types_1.RelativeDateDuration.WEEKS: {
            switch (direction) {
                case bf_types_1.RelativeDateDirection.BEFORE_TODAY: {
                    const currentDay = date.getDate();
                    date.setDate(currentDay - quantifier * 7);
                    break;
                }
                case bf_types_1.RelativeDateDirection.AFTER_TODAY: {
                    const currentDay = date.getDate();
                    date.setDate(currentDay + quantifier * 7);
                    break;
                }
            }
            break;
        }
        case bf_types_1.RelativeDateDuration.MONTH:
        case bf_types_1.RelativeDateDuration.MONTHS: {
            switch (direction) {
                case bf_types_1.RelativeDateDirection.BEFORE_TODAY: {
                    const currentMonth = date.getMonth();
                    date.setMonth(currentMonth - quantifier);
                    break;
                }
                case bf_types_1.RelativeDateDirection.AFTER_TODAY: {
                    const currentMonth = date.getMonth();
                    date.setMonth(currentMonth + quantifier);
                    break;
                }
            }
            break;
        }
        case bf_types_1.RelativeDateDuration.YEAR:
        case bf_types_1.RelativeDateDuration.YEARS: {
            switch (direction) {
                case bf_types_1.RelativeDateDirection.BEFORE_TODAY: {
                    const currentYear = date.getFullYear();
                    date.setFullYear(currentYear - quantifier);
                    break;
                }
                case bf_types_1.RelativeDateDirection.AFTER_TODAY: {
                    const currentYear = date.getFullYear();
                    date.setFullYear(currentYear + quantifier);
                    break;
                }
            }
            break;
        }
    }
    return {
        type: bf_types_1.ConfigurableDateType.STATIC,
        value: date.toISOString(),
    };
}
exports.relativeDateToStatic = relativeDateToStatic;
function getConfigureDateISOString(date) {
    if (isRelativeDate(date)) {
        return relativeDateToStatic(date).value;
    }
    return date.value;
}
exports.getConfigureDateISOString = getConfigureDateISOString;
function getConfigurableDateString(date) {
    let isoDateString = date.value;
    if (isRelativeDate(date)) {
        isoDateString = relativeDateToStatic(date).value;
    }
    const dateStringChunks = isoDateString.split('T')[0].split('-');
    const dateString = `${dateStringChunks[1]}/${dateStringChunks[2]}/${dateStringChunks[0]}`;
    return dateString[0] === '0' ? dateString.substr(1) : dateString;
}
exports.getConfigurableDateString = getConfigurableDateString;
