"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.temporalDateRangeFilter = exports.rangeFilter = exports.notInListFilter = exports.notEqualFilter = exports.notContainsInListFilter = exports.likeTextFilter = exports.lessThanOrEqualFilter = exports.lessThanFilter = exports.inListFilter = exports.greaterThanOrEqualFilter = exports.greaterThanFilter = exports.exactMatchFilter = exports.containsInListFilter = exports.containsLikeTextFitler = exports.containsFilter = exports.searchFilters = exports.searchFilter = void 0;
const bf_types_1 = require("bf-types");
function searchFilter(fieldName, fieldValue) {
    return {
        field_name: fieldName,
        field_value: fieldValue,
    };
}
exports.searchFilter = searchFilter;
function searchFilters(...filters) {
    return filters.map(searchFilterPairToObject);
}
exports.searchFilters = searchFilters;
exports.containsFilter = makeFilter(bf_types_1.SearchFilterType.CONTAINS)()(fromPrimitive);
exports.containsLikeTextFitler = makeFilter(bf_types_1.SearchFilterType.CONTAINS_LIKE_TEXT)()(fromString);
exports.containsInListFilter = makeFilter(bf_types_1.SearchFilterType.CONTAINS_IN_LIST)()(fromPrimitives);
exports.exactMatchFilter = makeFilter(bf_types_1.SearchFilterType.EXACT_MATCH)()(fromPrimitive);
exports.greaterThanFilter = makeFilter(bf_types_1.SearchFilterType.GREATER_THAN)()(fromPrimitive);
exports.greaterThanOrEqualFilter = makeFilter(bf_types_1.SearchFilterType.GREATER_THAN_OR_EQUAL)()(fromPrimitive);
exports.inListFilter = makeFilter(bf_types_1.SearchFilterType.IN_LIST)()(fromPrimitives);
exports.lessThanFilter = makeFilter(bf_types_1.SearchFilterType.LESS_THAN)()(fromPrimitive);
exports.lessThanOrEqualFilter = makeFilter(bf_types_1.SearchFilterType.LESS_THAN_OR_EQUAL)()(fromPrimitive);
exports.likeTextFilter = makeFilter(bf_types_1.SearchFilterType.LIKE_TEXT)()(fromString);
exports.notContainsInListFilter = makeFilter(bf_types_1.SearchFilterType.NOT_CONTAINS_IN_LIST)()(fromPrimitives);
exports.notEqualFilter = makeFilter(bf_types_1.SearchFilterType.NOT_EQUAL)()(fromPrimitive);
exports.notInListFilter = makeFilter(bf_types_1.SearchFilterType.NOT_IN_LIST)()(fromPrimitives);
exports.rangeFilter = makeFilter(bf_types_1.SearchFilterType.RANGE)()(fromRange);
exports.temporalDateRangeFilter = makeFilter(bf_types_1.SearchFilterType.TEMPORAL_DATE_RANGE)()(fromTemporalRange);
function makeFilter(type) {
    return () => (run) => {
        return (fieldName, ...params) => ({
            field_name: fieldName,
            field_value: withType(type)(run(...params)),
        });
    };
}
function fromPrimitive(value, constraint) {
    return { value, constraint };
}
function fromPrimitives(values, constraint) {
    return { values, constraint };
}
function fromString(value, constraint) {
    return { value, constraint };
}
function fromRange(start, end) {
    return { start, end };
}
function fromTemporalRange(start, end, constraint) {
    return { start, end, temporal_constraint: constraint };
}
function searchFilterPairToObject(filter) {
    return Array.isArray(filter) ? searchFilter(filter[0], filter[1]) : filter;
}
function withType(type) {
    return (value) => (Object.assign({ type }, value));
}
