"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function eventBus() {
    const eventHandlerMap = new Map();
    function publish(channel, event) {
        const eventHandlers = eventHandlerMap.get(channel);
        if (!eventHandlers) {
            return;
        }
        for (const handle of eventHandlers) {
            handle(event);
        }
    }
    function subscribe(channel, handler) {
        let eventHandlers = eventHandlerMap.get(channel);
        if (!eventHandlers) {
            eventHandlers = new Set();
            eventHandlerMap.set(channel, eventHandlers);
        }
        eventHandlers.add(handler);
    }
    const bus = {
        publish,
        subscribe,
    };
    return Object.freeze(bus);
}
exports.default = eventBus;
