"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetInstrumentorLogger = void 0;
let getTime = () => -1;
function load(moduleName) {
    // eslint-disable-next-line no-eval
    return eval('require')(moduleName);
}
let logger = () => { };
const Instrumentor = function Instrumentor() {
    getTime = typeof window === 'undefined' ? load('perf_hooks').performance.now : performance.now;
    this.metricScopes = new Map([['default', 0]]);
    this.metricLabels = new Map([['default', 0]]);
    this.sessionName = '';
    this.sessionStart = Date.now();
    this.sessionStart = -1;
    return this;
};
exports.default = Instrumentor;
Instrumentor.prototype.beginSession = function beginSession(name, options = {}) {
    logger('debug', `starting a new instrumentor session named: ${name}`);
    this.sessionName = name;
    this.sessionStart = getTime();
    if (!options.keepLabels) {
        this.metricLabels = new Map([['default', 0]]);
    }
    if (!options.keepScopes) {
        this.metricScopes = new Map([['default', 0]]);
    }
    if (!options.keepResults) {
        this.results = [];
    }
};
Instrumentor.prototype.endSession = function endSession() {
    logger('debug', `ending an active instrumentor session named: ${this.sessionName}`);
    const result = {
        metricLabels: strMapToStrArr(this.metricLabels),
        metricScopes: strMapToStrArr(this.metricScopes),
        sessionName: this.sessionName,
        sessionStart: this.sessionStart,
        sessionEnd: getTime(),
        results: Array.from(this.results),
    };
    return result;
};
Instrumentor.prototype.newLabel = function newLabel(name) {
    logger('debug', `added a new instrumentor label named: ${name}`);
    let value = this.metricLabels.get(name);
    if (value) {
        logger('warn', `tried to add a duplicate instrumentor label named: ${name}`);
        return value;
    }
    value = this.metricLabels.size;
    this.metricLabels.set(name, value);
    return value;
};
Instrumentor.prototype.newScope = function newScope(name) {
    logger('debug', `added a new instrumentor scope named: ${name}`);
    let value = this.metricScopes.get(name);
    if (value) {
        logger('warn', `tried to add a duplicate instrumentor scope named: ${name}`);
        return value;
    }
    value = this.metricScopes.size;
    this.metricScopes.set(name, value);
    return value;
};
Instrumentor.prototype.metric = function metric(options = {}) {
    const scopeName = options.scope || 'default';
    let scopeValue = this.metricScopes.get(scopeName);
    if (!scopeValue) {
        scopeValue = this.newScope(scopeName);
    }
    const labelName = options.label || 'default';
    let labelValue = this.metricLabels.get(labelName);
    if (!labelValue) {
        labelValue = this.newLabel(labelName);
    }
    const collector = new MetricCollector(scopeValue, labelValue, (result) => this.results.push(result));
    return collector;
};
/* ~~~ Metric Collector Definitions ~~~ */
const MetricCollector = function MetricCollector(scope, label, onSubmit) {
    this.labelValue = label;
    this.scopeValue = scope;
    this.onSubmit = onSubmit;
    return this;
};
MetricCollector.prototype.start = function start() {
    const start = getTime();
    return () => this.onSubmit({
        label_id: this.labelValue,
        scope_id: this.scopeValue,
        start,
        end: getTime(),
    });
};
/* ~~~ Local Utils ~~~ */
function strMapToStrArr(map) {
    const strs = [];
    for (const [key, index] of map.entries()) {
        strs[index] = key;
    }
    return strs;
}
/* ~~~ System Utils ~~~ */
function SetInstrumentorLogger(newLogger) {
    logger('debug', 'replacing the instrumentor logger with a new one');
    if (typeof newLogger === 'function') {
        logger = newLogger;
    }
}
exports.SetInstrumentorLogger = SetInstrumentorLogger;
