"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
function nexus(system, config, clientAuth) {
    return __awaiter(this, void 0, void 0, function* () {
        let user = null;
        let baseUrl = config.url;
        if (baseUrl[baseUrl.length - 1] === '/') {
            baseUrl = baseUrl.slice(0, -1);
        }
        function getUrl() {
            return baseUrl;
        }
        function getUser() {
            return user;
        }
        function getLoginUrl(returnUrl) {
            if (typeof returnUrl !== 'string') {
                return `${baseUrl}/login`;
            }
            return `${baseUrl}/login?returnUrl=${returnUrl}`;
        }
        function disconnect() {
            return __awaiter(this, void 0, void 0, function* () {
                yield clientAuth.disconnect(system);
                if (clientAuth.afterDisconnect) {
                    clientAuth.afterDisconnect(system);
                }
            });
        }
        function reconnect() {
            return __awaiter(this, void 0, void 0, function* () {
                yield disconnect();
                const result = yield clientAuth.reconnect(system);
                if (!result) {
                    return null;
                }
                user = result;
                if (clientAuth.afterConnect) {
                    clientAuth.afterConnect(system);
                }
                return result;
            });
        }
        const instance = {
            getUrl,
            getLoginUrl,
            disconnect,
            reconnect,
        };
        try {
            user = yield clientAuth.connect(baseUrl);
            if (!user) {
                throw new Error('was not able to authenticate the user');
            }
            return Object.freeze(Object.assign(Object.assign({}, instance), { getUser }));
        }
        catch (e) {
            throw new Error(`failed to establish a proper connection with a nexus node: ${e.message}`);
        }
    });
}
exports.default = nexus;
